@@ -39,7 +39,7 @@ _boxed_dealloc (PyGIBoxed *self)
         }
     }
 
-    Py_TYPE( (PyGObject *) self)->tp_free ( (PyObject *) self);
+    Py_TYPE (self)->tp_free ((PyObject *)self);
 }
 
 void *
